<?php

/**
 * @file
 * Salespush admin routines.
 */

/**
 * Salespush configuration form.
 */
function salespush_admin($form, &$form_state) {
  $form['salespush_login'] = array(
    '#type' => 'textfield',
    '#title' => t('Login for production salesforce.com'),
    '#default_value' => variable_get('salespush_login'),
    '#attributes' => array(
      'placeholder' => t('enter your production salesforce login'),
    ),
  );
  $form['salespush_pass'] = array(
    '#type' => 'password',
    '#title' => t('Password for production salesforce.com'),
    '#default_value' => variable_get('salespush_pass'),
    '#attributes' => array(
      'placeholder' => t('enter your production salesforce password'),
    ),
  );
  $form['salespush_token'] = array(
    '#type' => 'password',
    '#title' => t('Security token for production salesforce.com'),
    '#default_value' => variable_get('salespush_token'),
    '#attributes' => array(
      'placeholder' => t('enter your production salesforce security token'),
    ),
  );

  $form['salespush_prod_wsdl'] = array(
    '#type' => 'select',
    '#title' => t('WSDL file for production salesforce.com'),
    '#options' => salespush_wsdls(),
    '#default_value' => variable_get('salespush_prod_wsdl'),
    '#description' => t('Select wsdl file from salespush wsdl directory.'),
  );

  $form['salespush_sandbox_login'] = array(
    '#type' => 'textfield',
    '#title' => t('Login for sandbox test.salesforce.com'),
    '#default_value' => variable_get('salespush_sandbox_login'),
    '#attributes' => array(
      'placeholder' => t('enter your sanbox salesforce login'),
    ),
  );
  $form['salespush_sandbox_pass'] = array(
    '#type' => 'password',
    '#title' => t('Password for sandbox test.salesforce.com'),
    '#default_value' => variable_get('salespush_sandbox_pass'),
    '#attributes' => array(
      'placeholder' => t('enter your sandbox salesforce password'),
    ),
  );
  $form['salespush_sandbox_token'] = array(
    '#type' => 'password',
    '#title' => t('Security token for sandbox test.salesforce.com'),
    '#default_value' => variable_get('salespush_sandbox_token'),
    '#attributes' => array(
      'placeholder' => t('enter your salesforce sandbox security token'),
    ),
  );

  $form['salespush_sandbox_wsdl'] = array(
    '#type' => 'select',
    '#title' => t('WSDL file for sandbox test.salesforce.com'),
    '#options' => salespush_wsdls(),
    '#default_value' => variable_get('salespush_sandbox_wsdl'),
    '#description' => t('Select wsdl file from salespush wsdl directory.'),
  );

  return system_settings_form($form);
}

/**
 * Get list of available wsdls in salespush/wsdl directory.
 *
 * @return array
 *   Array for #options element.
 */
function salespush_wsdls() {
  // Get list of files from /wsdl dir in salespush module root.
  $wsdl_path = drupal_get_path('module', 'salespush') . '/wsdl';
  $wsdl_dir_content = array_diff(scandir($wsdl_path), array('..', '.'));
  // Create array('filename' => 'filename') for #options data.
  $options = array_combine($wsdl_dir_content, $wsdl_dir_content);

  return $options;
}

/**
 * Salespush sandbox content form.
 */
function salespush_content($form, &$form_state) {
  $form['salespush_stats'] = array(
    '#type' => 'textarea',
    '#title' => t('Last job log'),
    '#default_value' => variable_get('salespush_sandbox_lastlogs', t('Data not yet pushed.')),
    '#disabled' => TRUE,
    '#rows' => 25,
  );
  $form['salespush_push'] = array(
    '#type' => 'submit',
    '#value' => t('Push users to sandbox test.salesforce.com'),
    '#weight' => -10,
  );

  $form['salespush_push_demo'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('salespush_push_demo'),
    '#title' => t('Use demo data user DemoXXX, not real users database.'),
    '#weight' => -11,
  );

  return $form;
}

/**
 * Salespush production content form.
 */
function salespush_content_production($form, &$form_state) {
  $form['salespush_prod_stats'] = array(
    '#type' => 'textarea',
    '#title' => t('Last job log'),
    '#default_value' => variable_get('salespush_production_lastlogs', t('Data not yet pushed.')),
    '#disabled' => TRUE,
    '#rows' => 25,
  );
  $form['salespush_prod_push'] = array(
    '#type' => 'submit',
    '#value' => t('Push users to production salesforce.com'),
    '#weight' => -10,
  );
  $form['salespush_push_demo'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('salespush_push_demo'),
    '#title' => t('Use demo data user DemoXXX, not real users database.'),
    '#weight' => -11,
  );

  return $form;
}

/**
 * Submit handler for sandbox salesforce.
 */
function salespush_content_submit($form, &$form_state) {

  // Setup DemoXXX user content.
  if ($form_state['values']['salespush_push_demo'] == 0) {
    variable_set('salespush_push_demo', FALSE);
  }
  else {
    variable_set('salespush_push_demo', TRUE);
  }

  variable_del('salespush_sandbox_lastlogs');

  // Salesforce Login information.
  // This wsdl was manually generated using Setup/Develop/API menu in web UI.
  // Generate Enterprise WSDL link.
  $wsdl = drupal_get_path('module', 'salespush') . '/wsdl/' . variable_get('salespush_sandbox_wsdl');

  $username = variable_get('salespush_sandbox_login');
  $password = variable_get('salespush_sandbox_pass') . variable_get('salespush_sandbox_token');

  if (variable_get('salespush_push_demo')) {
    $last_logs = _salespush_pusher('sandbox', $wsdl, $username, $password);
  }
  else {
    $last_logs = _salespush_pusher('sandbox', $wsdl, $username, $password, FALSE);
  }

  variable_set('salespush_sandbox_lastlogs', $last_logs);
}

/**
 * Submit handler for production salesforce.
 */
function salespush_content_production_submit($form, &$form_state) {

  // Setup DemoXXX user content.
  if ($form_state['values']['salespush_push_demo'] == 0) {
    variable_set('salespush_push_demo', FALSE);
  }
  else {
    variable_set('salespush_push_demo', TRUE);
  }

  variable_del('salespush_production_lastlogs');

  // Salesforce Login information.
  // This wsdl was manually generated using Setup/Develop/API menu in web UI.
  // Generate Enterprise WSDL link.
  $wsdl = drupal_get_path('module', 'salespush') . '/wsdl/' . variable_get('salespush_prod_wsdl');

  $username = variable_get('salespush_login');
  $password = variable_get('salespush_pass') . variable_get('salespush_token');

  if (variable_get('salespush_push_demo')) {
    $last_logs = _salespush_pusher('production', $wsdl, $username, $password);
  }
  else {
    $last_logs = _salespush_pusher('production', $wsdl, $username, $password, FALSE);
  }

  variable_set('salespush_production_lastlogs', $last_logs);
}

/**
 * Helper function for sending BULK API csv data.
 *
 * @param string $environment
 *   Environment: 'production', 'sandbox' or other.
 * @param string $wsdl
 *   Full path to wsdl.xml file.
 * @param string $username
 *   Salesforce username.
 * @param string $password
 *   Salesforce password.security_token.
 * @param bool $demo
 *   Enable/disable demo data.
 *
 * @return string
 *   Return log result
 */
function _salespush_pusher($environment, $wsdl, $username, $password, $demo = TRUE) {

  require_once 'vendor/developerforce/force.com-toolkit-for-php/soapclient/SforceEnterpriseClient.php';
  $client = new SforceEnterpriseClient();
  $client->createConnection($wsdl);

  $login_result = $client->login($username, $password);
  $session_id = $client->getSessionId();
  $location = $client->getLocation();

  watchdog('SF ' . $environment . ' session_id:', print_r($session_id, TRUE));
  watchdog('SF ' . $environment . ' location:', print_r($location, TRUE));

  require_once 'vendor/ryanbrainard/forceworkbench/workbench/bulkclient/BulkApiClient.php';
  $my_bulk_api_connection = new BulkApiClient($location, $session_id);
  // Optional, but using here for demo purposes.
  $my_bulk_api_connection->setLoggingEnabled(TRUE);
  // Optional, but recommended. Defaults to true.
  $my_bulk_api_connection->setCompressionEnabled(TRUE);

  // STEP 3: CREATE A NEW JOB.
  // Create in-memory representation of the job.
  $job = new JobInfo();
  $job->setObject("Contact");
  $job->setOpertion("upsert");
  // This is a unique field for data syncing (upserting).
  // Old data will be overwrited.
  // Rows without email will be skipped by salesforce.
  $job->setExternalIdFieldName('Email');
  $job->setContentType("CSV");
  // Can be also set to Serial.
  $job->setConcurrencyMode("Parallel");

  // Send the job to the Bulk API and pass back returned JobInfo to the same variable.
  $job = $my_bulk_api_connection->createJob($job);

  // STEP 4. CREATE A NEW BATCH
  // prep the data. normally this would be loaded from a file,
  // but showing in plain text for demo purposes.
  if ($demo) {
    $csv_data = "\"FirstName\",\"LastName\",\"Email\"\n" .
      "\"DemoXXX\",\"User\",\"demo@example.com\"\n";
  }
  else {
    $csv_data = salespush_get_users();
  }
  $batch = $my_bulk_api_connection->createBatch($job, $csv_data);

  // STEP 5. CLOSE THE JOB.
  $my_bulk_api_connection->updateJobState($job->getId(), "Closed");

  // STEP 6: MONITOR BATCH STATUS UNTIL DONE.
  $i = 0;
  while ($batch->getState() == "Queued" || $batch->getState() == "InProgress" && $i < 3) {
    $batch = $my_bulk_api_connection->getBatchInfo($job->getId(), $batch->getId());
    // Wait for 5 seconds before polling again.
    sleep(5);
    $i++;
  }

  // STEP 7: GET BATCH RESULTS.
  $batch_results = $my_bulk_api_connection->getBatchResults($job->getId(), $batch->getId());

  // LOGGING EVERYTHING THAT HAPPENED ABOVE.
  watchdog('SF ' . $environment . ' BATCH RESULTS', htmlspecialchars($batch_results));
  $result = $my_bulk_api_connection->getLogs();
  watchdog('SF ' . $environment . ' CLIENT LOGS', $result);
  // Clear log buffer.
  $my_bulk_api_connection->clearLogs();

  return htmlspecialchars_decode($result);
}

/**
 * Get users for push into SalesForce.
 *
 * @return string
 *   Return string representation of CSV file.
 */
function salespush_get_users() {

  /* Field for pushing to SalesForce
  Name -> FirstName + LastName
  photo -> no field!!!
  headline -> Title
  current positions -> no field!!!
  Emails -> Email
  */
  // SELECT * FROM `users` WHERE `mail` LIKE '%@%' AND `data` LIKE '%a:%' LIMIT 50000.
  $users_query = db_select('users', 'u');
  $users_query = $users_query->fields('u', array('mail', 'data'));
  $users_query = $users_query->condition('mail', '%@%', 'LIKE');
  $users_query = $users_query->condition('data', 'a:%', 'LIKE');
  // SalesForce limit 50000 records at once.
  $users_query = $users_query->range(0, 50000);
  $users_query = $users_query->execute();

  $csvcontent = "\"FirstName\",\"LastName\",\"Email\",\"Title\"\n";
  foreach ($users_query as $user) {
    if (isset($user->data)) {
      $userdata = unserialize($user->data);
      // Do not process if there is no basic_profile in data array.
      if (!isset($userdata['basic_profile'])) {
        continue;
      }
      // Create a line for csv.
      $firstname = isset($userdata['basic_profile']['first-name']) ? $userdata['basic_profile']['first-name'] : '#N/A';
      $lastname = isset($userdata['basic_profile']['last-name']) ? $userdata['basic_profile']['last-name'] : '#N/A';
      $email = $user->mail;
      $title = isset($userdata['basic_profile']['headline']) ? $userdata['basic_profile']['headline'] : '#N/A';

      // Escaping strings to be ready for CSV export.
      $firstname = str_replace('"', '""', $firstname);
      $lastname = str_replace('"', '""', $lastname);
      $title = str_replace('"', '""', $title);

      $csvcontent .= "\"$firstname\",\"$lastname\",\"$email\",\"$title\"\n";
    }
  }

  return $csvcontent;
}
